#include "SysMainWindow.h"
#include <QCloseEvent>
#include <QDesktopServices>
#include <QProcess>
#include <QGraphicsDropShadowEffect>
#include <QHBoxLayout>
#include <QMenu>
#include <QDateTime>
#include <QResource>
#include <QTabBar>
#include <QFileDialog>
#include <QFileInfo>
#include <QToolBar>
#include <QApplication>
#include <QVBoxLayout>
#include <QPainter>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tfileio.h>
#include <tbaseutil/tlogger.h>
#include <tbaseutil/tresource.h>
#include <tbaseutil/ttheme.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <toputil/topaboutusdialog.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topcore.h>
#include <toputil/toploginpwdrestdialog.h>
#include <topcore/topmessagecontroller.h>
#include <twidget/tmessagebox.h>
#include <twidget/tpushbutton.h>
#include <twidget/tuiloaderdialog.h>
#include <twidget/tpanelmenu.h>
#include <twidget/twebview.h>
#include <twidget/ttoolbutton.h>
#include <twidget/tdialog.h>
#include <twidget/tradiobox.h>
#include <twidget/tcheckbox.h>
#include <twidget/tgroupbox.h>
#include <twidget/tdialogbuttonbox.h>




SysMainWindow::SysMainWindow(const QString &iModuleNameStr,
                   const QVariantMap &iUrlPars,
                   QWidget *iParent)
    :TopClassAbs(iParent)
{
    initModule(iModuleNameStr,iUrlPars);
    if (this->isHookExists("afterModuleInit")) {
            this->callHooks("afterModuleInit");
    }

    setHasWindowTitleBar(false);

       QStringList rccLst = config("resource.rcc").toStringList();
       for (QString rcc : rccLst) {
           TRES->loadRcc(rcc);
       }

       initMainWidget();

       setMinimumSize(TTHEME_DP(800), TTHEME_DP(600));
       QVariantMap defalutSize = config("default_size").toMap();
       if (!defalutSize.isEmpty() && defalutSize["width"].toInt() > 0
               && defalutSize["height"].toInt() > 0) {
           resize(TTHEME_DP(defalutSize["width"].toInt()),
                   TTHEME_DP(defalutSize["height"].toInt()));
       } else {
           showMaximized();
       }

       refreshActionState();
}

SysMainWindow::~SysMainWindow()
{
    if (this->isHookExists("onDestory")) {
        this->callHooks("onDestory");
    }
}

//初始化主界面
void SysMainWindow::initMainWidget()
{
    mBodySplitter = new QSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);
    mMgtSplitter = new QSplitter(this);
    mBodySplitter->addWidget(mMgtSplitter);

    mBodyWgt = new QWidget(this);
    QVBoxLayout *bodyLayout = new QVBoxLayout(mBodyWgt);
    bodyLayout->setMargin(0);
    bodyLayout->setSpacing(0);
    mMgtSplitter->addWidget(mBodyWgt);
}

//打开当前选择模块
void SysMainWindow::openModule(const QString &iUrl,
                          const QVariant &iConfig)
{
    bool exists = false;
    TopClassAbs *module = APP->openModule(iUrl, &exists);
    if (module == nullptr) {
        TLOG_ERROR(QString("open module(%1) failed!").arg(iUrl));
        return;
    }

    QVariantMap paramMap = iConfig.toMap();
    QString title = paramMap.value("title_" + APP->language()).toString();
    if (title.isEmpty()) {
        title = paramMap.value("title").toString();
    }
    module->setTitle(title);
    module->setIconName(paramMap.value("icon").toString());

    const QString url = module->url().toUpper();
    if (!exists) {
        connect(module, SIGNAL(windowTitleChanged(QString)),
                this, SLOT(onModuleWindowTitleChanged()),
                Qt::UniqueConnection);
        connect(module, SIGNAL(windowModifyChanged(bool)),
                this, SLOT(onModuleWindowTitleChanged()),
                Qt::UniqueConnection);
    }

    if (!mUrlAddressWidgetMap.contains(url)) {
        mUrlAddressWidgetMap.insert(url, module);
        mUrlAddressConfigMap.insert(url, iConfig.toMap());
    }
    QVariantMap config;
    config.insert("title", module->title());
    config.insert("icon", module->iconName());
    mStackedWidget->addWidget(module);
    mStackedWidget->setCurrentWidget(module);
}


void StackedWidget::setBackground(const QString &iBackground)
{
    mBackground = iBackground;
}

void StackedWidget::paintEvent(QPaintEvent *iEvent)
{
    Q_UNUSED(iEvent)

    if (!mBackground.isEmpty()) {
        QPainter p(this);
        p.drawPixmap(this->rect(), QPixmap(mBackground));
    }
}
