﻿#include "musermgt.h"
#include <QGraphicsDropShadowEffect>
#include <QToolBar>
#include <QVBoxLayout>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tenumlist.h>
#include <tbaseutil/ttheme.h>
#include <tbaseutil/tlogger.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <topcore/topcore.h>
#include <toputil/t.h>
#include <twidget/ttableview.h>
#include <twidget/tsearchentry.h>
#include <twidget/tpagetool.h>
#include <twidget/tadvancedquery.h>
#include <twidget/tmessagebar.h>
#include <twidget/tcategorytreeview.h>
#include <twidget/ttableviewdialog.h>
#include <twidget/tmessagebox.h>
#include <twidget/taccordion.h>
#include <twidget/tsplitter.h>
#include <twidget/tscrollarea.h>
#include <twidget/ttabwidget.h>
#include <tbaseutil/tresource.h>


Musermgt::Musermgt(const QString &iModuleNameStr,
                               const QVariantMap &iUrlPars,
                               QWidget *iParent):
    TopClassAbs(iParent)
{
    initModule(iModuleNameStr, iUrlPars);
    QString licenseKey = config("sys_common").toString();
    if (!licenseKey.isEmpty()) {
        setLicenseKey(licenseKey);
    }
    QStringList langLst = config("lang.conf").toStringList();
    for (const auto &lang : langLst) {
        appendLanguage(lang);
    }
    if (isHookExists("afterModuleInit")) {
        callHooks("afterModuleInit");
    }
    TopClassHelper::parseTableConf0(this, "view", &mTableConf);
    initTableView();
    connect(this, SIGNAL(detailSaved(QString)),
            this, SLOT(onDetailSaved(QString)));
    connect(this, SIGNAL(isDetailModifiedChanged(bool)),
            this, SLOT(onDetailChanged(bool)));
    connect(this, SIGNAL(detailUidChanged(QString)),
            this, SLOT(refreshDetail(QString)));
    if (isHookExists("afterViewInit")) {
        callHooks("afterViewInit");
    }
    restoreSizeState();
    restoreObjectState(mBodySplitter);
    restoreObjectState(mTableView);

    refreshActionState();
    QTimer::singleShot(0, this, SLOT(refresh()));

    auto refreshInterval = config("view.timing_refresh_interval", 0).toInt();
    if (refreshInterval > 0) {
        mRefreshTimer = new QTimer(this);
        connect(mRefreshTimer, &QTimer::timeout, [=]() {
            this->callAction("refresh");
        });
        mRefreshTimer->start(refreshInterval * 1000);
    }
}

Musermgt::~Musermgt()
{
    this->saveSizeState();
    this->saveObjectState(mTableView);
}

void Musermgt::reload()
{
    setDetailUid(mLastDetailUid.isEmpty() ? mDetailUid : mLastDetailUid, true);
}

bool Musermgt::DetailModified() const
{
    return IsDetailchange;
}

void Musermgt::setIsDetailModified(bool iIsDetailModified)
{
    if (IsDetailchange == iIsDetailModified) {
        qDebug()<<iIsDetailModified;
        return;
    }
    IsDetailchange = iIsDetailModified;
    emit isDetailModifiedChanged(iIsDetailModified);
}

QVariantList Musermgt::selectedItems()
{
    return mSelectedItems;
}

QString Musermgt::detailUid() const
{
   return mDetailUid;
}

void Musermgt::setDetailUid(const QString &iUid, bool iForceRefresh)
{
   if (iForceRefresh || mDetailUid != iUid) {
       mDetailUid = iUid;
       emit detailUidChanged(iUid);
    }
}

void Musermgt::newItem()
{
    mLastDetailUid = mDetailUid;
    setDetailUid(QString());
    setIsDetailModified(true);
}

//新增用户
void Musermgt::addUser()
{
    //验证有效性
    QVariantList errLst = DetailView->validateAll("COMMIT", true, "ERROR");
    if (!errLst.isEmpty()) {
        QStringList errStrLst;
        for (QVariant err : errLst) {
            errStrLst.append(err.toMap().value("text").toString());
        }
        alertError(ttr("saved failed!"), errStrLst.join("\n"));
        return;
    }

    //使用TopClassTableConf类对象配置各项信息
    TSqlInserterV2 insertor;
    insertor.setTable(mTableConf.dbTableName);
    QVariantMap saveData;
    QMap<QString, QString> updatePolicy;
    TopClassHelper::fetchSaveFieldsFromUi(DetailView, saveData, updatePolicy);
    insertor.addUniqueField(mTableConf.primaryKey);
    insertor.setAutoIncrementField(mTableConf.primaryKey);
    saveData.insert(mTableConf.primaryKey, mDetailUid);

    if (config("view.data_set.db_filter").type() == QVariant::Map) {
          QVariantMap dbFilterMap = config("view.data_set.db_filter").toMap();
          QMapIterator<QString, QVariant> i(dbFilterMap);
          while (i.hasNext()) {
              i.next();
              saveData.insert(i.key(), i.value());      //向saveData里插入各项数据的键和值
          }
      }
      insertor.setData(saveData);
      QStringList fields = saveData.keys();
      fields.removeOne(mTableConf.primaryKey);
      insertor.setField(fields);
      {
          QMapIterator<QString, QString> i(updatePolicy);
          while (i.hasNext()) {
              i.next();
              insertor.setUpdatePolicy(i.key(), i.value());
          }
      }
      TError err;
      QVariant result = runSqlQueryOnThreadSync(TOPSQLTHREAD_REPLACE_ROW,
                                                QVariant::fromValue(insertor),
                                                &err);
      if (err.isValid()) {
          alertError(ttr("Save data failed!"), err.text());
          return;
      }
      emit detailSaved(result.toString());
      setIsDetailModified(false);
      alertOk(ttr("Data saved"));
}

//删除用户
void Musermgt::removeUser(const QVariantList &iItems)
{
    if (mTableConf.dbDelFlagKey.isEmpty()) {
        TSqlDeleterV2 deleter;
        deleter.setTable(mTableConf.dbTableName);
        deleter.setWhere(mTableConf.primaryKey, iItems);
        TError err;
        runSqlQueryOnThreadSync(TOPSQLTHREAD_DELETE_ROW,
                                QVariant::fromValue(deleter),
                                &err);
        if (err.isValid()) {
            alertError(ttr("Delete data failed!"), err.text());
        } else {
            refresh(true);
            alertOk(ttr("Data deleted"));
        }
    } else {
        TSqlUpdaterV2 updater;
        updater.setTable(mTableConf.dbTableName);
        updater.setWhere(mTableConf.primaryKey, iItems);
        updater.setData(QVariantMap {{mTableConf.dbDelFlagKey, 1}});
        TError err;
        runSqlQueryOnThreadSync(TOPSQLTHREAD_UPDATE_ROW,
                                QVariant::fromValue(updater),
                                &err);
        if (err.isValid()) {
            alertError(ttr("Delete data failed!"), err.text());
        } else {
            refresh(true);
            alertOk(ttr("Moved to recycle bin"));
        }
    }
}

void Musermgt::onPageChanged()
{
    refresh(false);
}

void Musermgt::onSelectionChanged()
{
    mSelectedItems = mTableView->selectedPrimaryKeys();
    QString curId;
    if (!mSelectedItems.isEmpty()) {
        curId = mSelectedItems.first().toString();
    }
    if (mDetailUid != curId) {
        mLastDetailUid = curId;
        if (!IsDetailchange) {
            setDetailUid(curId);
        }
    }
    refreshActionState();
}

void Musermgt::onDetailSaved(const QVariant &iUidStr)
{
    this->refresh(false);
    mTableView->selectRow(iUidStr);
}

void Musermgt::onDetailDataChanged()
{
    setIsDetailModified(true);
}

void Musermgt::onDetailChanged(bool iIsDetailModified)
{   
    mTableView->setEnabled(!iIsDetailModified);
    if (iIsDetailModified) {
        showMaskFrame(mBodyWidget);
    } else {
        hideMaskFrame();
    }
    refreshActionState();
}

//刷新表格
void Musermgt::refresh(bool iResetPage)
{
    TSqlSelectorV2 selector;
    if (!mTableConf.dbSql.isEmpty()) {
        selector.setTable(QString("(1% TEMP_TABLE)").arg(mTableConf.dbSql));
    } else if (!mTableConf.dbTableName.isEmpty()) {
        selector.setTable(mTableConf.dbTableName);
    }
    selector.setField(mTableConf.queryFields);
    selector.setFieldFormat(mTableConf.fieldFormatMap);

    selector.setReturnRowCount(true);
    TopClassHelper::handleSearchPageOnQuery(mSearchEntry, mPageTool, iResetPage, &selector);
    TopClassHelper::handleOrderOnQuery(mTableView, &mTableConf, &selector);
    if (!mTableConf.dbDelFlagKey.isEmpty()) {
        selector.addWhere(QString("%1 = 0 OR %1 IS NULL").arg(mTableConf.dbDelFlagKey));
    }
    if(!mTableConf.dbFilter.isEmpty()) {
        selector.addWhere(mTableConf.dbFilter);
    }
    t::loading(this);
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
    unloading();
    TDataResponse response(data.toMap());
    if (response.hasError()) {
        alertError(ttr("Load data failed!"));
    } else {
        if (mPageTool != nullptr) {
            mPageTool->setRowCount(response.dataCount(), true);
        }
        QVariantList tableData = response.data().toList();
        TopClassHelper::formatTableData(this, &mTableConf, tableData);
        if (isHookExists("handleTableData")) {
            tableData = callHooksQuick("handleTableData", QVariantList() << QVariant(tableData)).toVariant().toList();
        }
        mTableView->loadData(tableData);
        mTableView->adjustViewSpan(mTableConf.mergeKeys);
        mSelectedItems.clear();
        setDetailUid(QString());
        alertOk(ttr("Data loaded"));
    }
    return;
}

//刷新详细表单
void Musermgt::refreshDetail(const QString &iUid)
{
    if (iUid.isEmpty()) {
        DetailView->loadValues(QVariantMap(), true);
    } else {
        TSqlSelectorV2 selector;
        if (!mTableConf.dbSql.isEmpty()) {
            selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTableConf.dbSql));
        } else if (!mTableConf.dbTableName.isEmpty()) {
            selector.setTable(mTableConf.dbTableName);
        } else {
            return;
        }
        selector.setField(mTableConf.queryFields);
        selector.setWhere(mTableConf.primaryKey, iUid);
        selector.setFieldFormat(mTableConf.fieldFormatMap);
        TError err;
        QVariant data = runSqlQueryOnThreadSync(TOPSQLTHREAD_SELECT_MAP,
                                                QVariant::fromValue(selector),
                                                &err);
        if (err.isValid()) {
            alertError(ttr("Load data failed!"), err.text());
            return;
        }
        QVariantMap dataMap = data.toMap();
        DetailView->loadValues(dataMap, true);
    }
    alertOk(ttr("Data loaded"));
    setIsDetailModified(false);
}

//初始化界面
void Musermgt::initTableView()
{
    mBodySplitter = new TSplitter(this);
    mBodySplitter->setObjectName("mBodySplitter");
    setCentralWidget(mBodySplitter);
    mBodyWidget = new TSplitter(this);
    mBodySplitter->addWidget(mBodyWidget);

    mBodyWgt = new QWidget(this);
    QVBoxLayout *bodyLayout = new QVBoxLayout(mBodyWgt);
    bodyLayout->setMargin(0);
    bodyLayout->setSpacing(0);
    mBodyWidget->addWidget(mBodyWgt);
    initNavi();

    mTableView = new TTableView(this);
    mTableView->setObjectName("__view__");
    mTableView->setHeaderPopupEnabled(true);
    connect(mTableView->horizontalHeader(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this, SLOT(refresh()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SIGNAL(selectionChanged()));
    mTableView->setDataKeyList(mTableConf.dataKeys);
    mTableView->setPrimaryKey(mTableConf.primaryKey);
    QVariantList headerItems = mTableConf.horizontalHeaders;
    headerItems.prepend(QVariant());
    mTableView->setHeaderItem(headerItems);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
        if (mSearchEntry != nullptr) {
            connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refresh()));
            mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mTableView->headerItem()));
            mSearchEntry->setPlaceholderText(ttr("Search %1"));
        }
    }
    if (QMenu *tablePopup = qobject_cast<QMenu *>(uim()->getWidget("TABLEVIEW_POPUP"))) {
        mTableView->setContextMenu(tablePopup);
    }

    bodyLayout->addWidget(mTableView, 1);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        bodyLayout->addWidget(toolbar, 0);
        mPageTool = qobject_cast<TPageTool *>(uim()->getWidget("BOTTOM_TOOLBAR/PAGE_TOOL"));
        if (mPageTool != nullptr) {
            mPageTool->setPageSizeVisible(true);
            connect(mPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onPageChanged()));
        }
    }

    QWidget *detailWgt = new QWidget(this);
    QVBoxLayout *detailLayout = new QVBoxLayout(detailWgt);
    detailLayout->setSpacing(0);
    detailLayout->setMargin(0);

    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("DETAIL_TOOLBAR"))) {
        detailLayout->addWidget(toolbar, 0);
    }
    DetailView = new TUiLoader(this);
    DetailView->setObjectName("__detail__");
    const QStringList typePlugins = config("uiloader_plugins").toStringList();
    if (typePlugins.size() > 0) {
        TopClassHelper::setUiLoaderPluginCallback(DetailView,this,typePlugins);
    }
    connect(DetailView, SIGNAL(dataChanged()), this, SLOT(onDetailDataChanged()));
    DetailView->setProperty("SS_BG", "PANEL");
    DetailView->setSelf(this);
    DetailView->setScriptEngine(APP->scriptEngine());
    DetailView->setUiStr(ui("detail").toString());
    detailLayout->addWidget(DetailView);

    if (!ui("detail").isNull()) {
        mBodySplitter->addWidget(detailWgt);
        mBodySplitter->setStretchFactor(0, 1);
        mBodySplitter->setSizes(QList<int>{1, config("detail.perfect_width", 600).toInt()});
    } else {
        detailWgt->setVisible(false);
    }
}

Musermgt::NaviType Musermgt::str2NaviType(const QString &iType)
{
    const QString type = iType.toUpper();
    if (type == "UILOADER") {
        return NaviType::UiLoader;
    } else if (type == "ADVANCEDQUERY") {
        return NaviType::AdvancedQuery;
    } else if (type == "NAVIFILTER") {
        return NaviType::NaviFilter;
    } else if (type == "CATEGORYTREEVIEWANDADVANCEDQUERY") {
        return NaviType::CategoryTreeViewAndAdvancedQuery;
    } else if (type == "NONE") {
        return NaviType::None;
    } else {
        return NaviType::CategoryTreeView;
    }
}

QVariantList Musermgt::parseCategoryConfig()
{
    QVariantList categoryLst;

    QVariantList categoryConfig = config("navi.categories").toList();
    QString format = config("navi.format", "filter_by_enum").toString();
    if (format.compare("filter_by_enum") == 0) {
        for(const QVariant &item: categoryConfig) {
            QVariantMap itemMap = item.toMap();
            QString itemName         = itemMap.value("name").toString();
            QVariantList childrenLst = itemMap.value("children").toList();
            QString enumCfg          = itemMap.value("enum_children").toString();
            QStringList enumCheckedLst = itemMap.value("enum_checked").toStringList();
            QStringList enumInvisibleLst = itemMap.value("enum_invisible").toStringList();
            if (itemMap.contains("visible")) {
                itemMap.insert("VISIBLE", itemMap.take("visible"));
            }
            if (itemMap.contains("expand")) {
                itemMap.insert("EXPAND", itemMap.take("expand"));
            }
            TSqlWhereCompsiteV2 childWhere;
            childWhere.setLogic(TSqlWhereCompsiteV2::Logic_Or);
            QVariantList type = TOPENM->enumList(enumCfg)->toComboList();
            for (const QVariant &item: type) {
                QVariantMap enumMap = item.toMap();
                QString enumName = enumMap.value("name").toString();
                QVariantMap childMap;
                childMap.insert("name",enumName);
                childMap.insert("text",ttr(enumMap.value("text").toString()));
                childMap.insert("icon",enumMap.value("icon"));
                childMap.insert("data", itemName);
                if(enumCheckedLst.contains(enumName)) {
                    childMap.insert("checked",1);
                } else {
                    childMap.insert("checked",0);
                }
                if (enumInvisibleLst.contains(enumName)) {
                    childMap.insert("VISIBLE", 0);
                } else {
                    childMap.insert("VISIBLE",1);
                }
                childrenLst.append(childMap);
            }
            int checkedChildLen = 0;
            for (QVariant &item : childrenLst) {
                QVariantMap itemMap = item.toMap();
                QString name = itemMap.value("name").toString();
                if(itemMap.value("checked").toInt() == 1) {
                    childWhere.append(itemName, name);
                    checkedChildLen++;
                }
                item = itemMap;
            }
            if (checkedChildLen == childrenLst.length()) {
                itemMap.insert("checked", 1);
            } else {
                itemMap.insert("checked", 0);
            }
            //mCategoryWhere.append(childWhere);

            itemMap.remove("children");
            itemMap.remove("enum_children");
            itemMap.remove("enum_checked");
            itemMap.remove("enum_invisible");
            itemMap.insert("text",ttr(itemMap.value("text").toString()));
            itemMap.insert("CHILDREN",childrenLst);

            categoryLst.append(itemMap);
        }
    } else if (format.compare("filter_by_sql_where") == 0) {
        return categoryConfig;
    }
    return categoryLst;
}

QVariantMap Musermgt::getDefaultQueryDataFromCfgMap()
{
    QVariantMap conditionMap = config("advance").toMap().value("condition").toMap();
    QVariantList valueList = conditionMap.value("value").toList();
    for (QVariant &var: valueList) {
        QVariantMap m = var.toMap();
        m.insert("title", ttr(m.value("title").toString()));
        var = m;
    }
    conditionMap.insert("value", valueList);
    return conditionMap;
}

void Musermgt::initAdvanceCfgMap()
{
    mAdvancedQueryCfgMap = config("advance").toMap();
    QVariantList queryItems = mAdvancedQueryCfgMap.value("advanced_items").toList();
    QVariantList newQueryItems;
    for (QVariant var : queryItems) {
        QVariantMap m = var.toMap();
        // 翻译title
        m.insert("title", ttr(m.value("title").toString()));
        // 转换枚举
        QVariantList optionList;
        QRegExp regExp("enum(.*)");
        if ((m.value("optionList").type() == QVariant::String) && (regExp.indexIn(m.value("optionList").toString()) != -1)) {
            if (regExp.capturedTexts().count() > 1) {
                optionList = TOPENM->enumList(regExp.capturedTexts().value(1).remove("(").remove(")"))->toComboList();
                m.insert("optionList", optionList);
            }
        }
        newQueryItems << m;
    }
    mAdvancedQueryCfgMap.insert("advanced_items", newQueryItems);
}

void Musermgt::initNavi()
{
    mNaviType = str2NaviType(config("navi.__type__", "CategoryTreeView").toString());
    if (mNaviType == NaviType::None) {
        return;
    }

    TAccordion *accordion = new TAccordion(this);
    accordion->setObjectName("__navi__");
    accordion->setProperty("SS_BG","NAVI");
    accordion->setMinimumWidth(TTHEME_DP(config("navi.min_width", 150).toInt()));
    accordion->setMaximumWidth(TTHEME_DP(config("navi.max_width", 300).toInt()));
    accordion->setAccordionLocation(TAccordion::AccordionLocation::Left);
    QVBoxLayout *naviLayout = new QVBoxLayout(accordion);
    naviLayout->setMargin(0);
    naviLayout->setSpacing(0);
    QLabel *naviLabel = new QLabel(tr("Navigation"));
    naviLabel->setProperty("SS_TYPE", "SUBHEADER");
    naviLayout->addWidget(naviLabel,0);
    naviLayout->addSpacing(TTHEME_DP(4));
    accordion->expandButton()->setFixedHeight(TTHEME_DP(40));
    accordion->expandButton()->setToolTip(tr("Show Navigation"));
    accordion->collapseButton()->setToolTip(tr("Hide Navigation"));
    accordion->setIsExpanded(config("navi.is_expanded", false).toBool());
    mBodyWidget->insertWidget(0,accordion);

    if (mNaviType == NaviType::UiLoader) {
        mNaviView = new TUiLoader(this);
        TUiLoader *filterUi = qobject_cast<TUiLoader*>(mNaviView);
        filterUi->setSelf(this);
        filterUi->setScriptEngine(APP->scriptEngine());
        filterUi->setUiStr(ui("navi").toString());

        QFrame *naviWgt = new QFrame(this);
        QVBoxLayout *leftLayout = new QVBoxLayout(naviWgt);
        leftLayout->addWidget(mNaviView);
        QHBoxLayout *leftBtnsLayout = new QHBoxLayout();
        leftLayout->addLayout(leftBtnsLayout);
        QPushButton *resetBtn = new QPushButton(this);
        connect(resetBtn, SIGNAL(clicked(bool)), filterUi, SLOT(clearValues()));
        leftBtnsLayout->addWidget(resetBtn);
        resetBtn->setText(tr("Reset"));
        resetBtn->setIcon(TRES->icon("rotate-right"));
        QPushButton *filterBtn = new QPushButton(this);
        connect(filterBtn, SIGNAL(clicked(bool)), this, SLOT(refresh()));
        leftBtnsLayout->addWidget(filterBtn);
        filterBtn->setText(tr("Filter"));
        filterBtn->setIcon(TRES->icon("filter"));
        leftLayout->addStretch(1);
        naviLayout->addWidget(naviWgt);
    } else if (mNaviType == NaviType::AdvancedQuery) {
        Q_ASSERT_X(false, "TopTemplateClass4", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::NaviFilter) {
        Q_ASSERT_X(false, "TopTemplateClass4", "This feature is not yet supported.");
    } else if (mNaviType == NaviType::CategoryTreeViewAndAdvancedQuery) {
        // tab1-CategoryTreeView
        TCategoryTreeView *categoryTreeView = new TCategoryTreeView(this);
        categoryTreeView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));

        // tab2-tab1-ToolBar
        QToolBar *toolbar = new QToolBar(this);
        toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        toolbar->setProperty("SS_SIZE", "SMALL");
        toolbar->setMovable(false);

        QAction *queryAction = toolbar->addAction(APP->getResIcon("search"), tr("Query"));
        connect(queryAction, SIGNAL(triggered()), this, SLOT(refresh()));
        toolbar->addSeparator();

        QAction* addAction = toolbar->addAction(APP->getResIcon("addto-circle-o"), tr("Add"));
        addAction->setToolTip(tr("Add query criteria"));
        //connect(addAction, SIGNAL(triggered()), this, SLOT(onAddActionTriggered()));

        QAction* clearAction = toolbar->addAction(APP->getResIcon("eraser"), tr("Clear"));
        clearAction->setToolTip(tr("Clear all query criteria"));
        //connect(clearAction, SIGNAL(triggered()), this, SLOT(onClearActionTriggered()));

        QAction* saveAsAction = toolbar->addAction(APP->getResIcon("save"), tr("Save As"));
        saveAsAction->setToolTip(tr("Save query criteria as file"));
        //connect(saveAsAction, SIGNAL(triggered()), this, SLOT(onSaveAsActionTriggered()));

        QAction* openActtion = toolbar->addAction(APP->getResIcon("folder-open-o"), tr("Open"));
        openActtion->setToolTip(tr("Open query criteria from file"));
        //connect(openActtion, SIGNAL(triggered()), this, SLOT(onOpenActtionTriggered()));

        QAction* saveAsCommonQueryAction = toolbar->addAction(APP->getResIcon("save-as"), tr("Save"));
        saveAsCommonQueryAction->setToolTip(tr("Save as a common query"));
        //connect(saveAsCommonQueryAction, SIGNAL(triggered()), this, SLOT(onSaveAsCommonQueryActionTriggered()));

        // tab2-tab1-AdvancedQuery
        TAdvancedQuery *advancedQuery = new TAdvancedQuery(this);
        advancedQuery->setObjectName("AdvancedQuery");
        initAdvanceCfgMap();
        advancedQuery->setItemList(mAdvancedQueryCfgMap.value("advanced_items").toList());
        advancedQuery->setData(getDefaultQueryDataFromCfgMap());

        TScrollArea *scrollArea = new TScrollArea(this);
        scrollArea->setWidgetResizable(true);
        scrollArea->setWidget(advancedQuery);

        // tab2-tab1
        QMainWindow *mainWindow = new QMainWindow(this);
        mainWindow->addToolBar(toolbar);
        mainWindow->setCentralWidget(scrollArea);

        // tab2-tab2-TableView
        TTableView *commonQueryTable = new TTableView(this);
        commonQueryTable->setObjectName("CommonQueryTable");
        commonQueryTable->setShowGrid(false);
        commonQueryTable->setSelectionMode(QAbstractItemView::ExtendedSelection);
        commonQueryTable->verticalHeader()->setVisible(false);
        QVariantList headerItem;
        headerItem << QVariant() << TDataParse::variantList2Map(
                          QVariantList() <<"name" << "title" << "display" << tr("Query Name") << "resizeMode" << "Stretch" << "displayRole" << "$title");
        commonQueryTable->setHeaderItem(headerItem);
        commonQueryTable->setDataKeyList(QStringList() << "title" << "condition");
        commonQueryTable->setPrimaryKey("condition");
        //connect(commonQueryTable, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(onCommonQueryTableDoubleClicked(const QModelIndex &)));

        QAction *deleleCommonQueryActtion = new QAction(APP->getResIcon("remove"), tr("Delete query"), this);
        connect(deleleCommonQueryActtion, SIGNAL(triggered()), this, SLOT(onDeleleCommonQueryActtionTriggered()));
        QMenu *contextMenu = new QMenu(this);
        contextMenu->addAction(deleleCommonQueryActtion);
        commonQueryTable->setContextMenu(contextMenu);

        // 刷新常用查询表格数据
        //initCommonQueryMap();
        //commonQueryTable->loadData(mAdvancedQueryTableDataMap.value("advance").toList());

        // 高级查询Tab
        TTabWidget *advancedQueryTabWidget = new TTabWidget(this);
        advancedQueryTabWidget->setObjectName("AdvancedQueryTabWidget");
        advancedQueryTabWidget->addTab(mainWindow, tr("Advanced Query"));
        advancedQueryTabWidget->addTab(commonQueryTable, tr("Common Query"));

        // 导航Tab
        mNaviView = new TTabWidget(this);
        TTabWidget *tabWidget = qobject_cast<TTabWidget *>(mNaviView);
        tabWidget->addTab(categoryTreeView, tr("Category"));
        tabWidget->addTab(advancedQueryTabWidget, tr("Advanced Query"));
        naviLayout->addWidget(mNaviView);
    } else {
        mNaviView = new TCategoryTreeView(this);
        mNaviView->setStyleSheet("TCategoryTreeView{border:NONE;background-color:transparent}");
        QVariantList rootChildList = parseCategoryConfig();
        if (rootChildList.isEmpty()) {
            mNaviView->setVisible(false);
            return;
        }
        TCategoryTreeView *categoryTreeView = qobject_cast<TCategoryTreeView *>(mNaviView);
        categoryTreeView->setItemList(rootChildList);
        categoryTreeView->setCheckable(config("navi.is_checkable").toBool());
        connect(categoryTreeView, SIGNAL(selectionChanged(QVariantList)),
                this, SLOT(onCategoryViewDataChanged(QVariantList)));
        naviLayout->addWidget(mNaviView);
    }
}

